
-----------------------
---- BEGIN: CREATE USER
-----------------------

CREATE user SECURPFC IDENTIFIED BY PFCBDREL_2015
DEFAULT TABLESPACE USERS
QUOTA UNLIMITED ON USERS
TEMPORARY TABLESPACE TEMP;

GRANT CREATE SESSION, CREATE TABLE, CREATE SEQUENCE, CREATE PROCEDURE, ALTER SESSION TO SECURPFC;

-----------------------
---- END  : CREATE USER
-----------------------

---------------------------
---- BEGIN: CREATE SEQUENCE
---------------------------

CREATE SEQUENCE SECURPFC.SEQ_LOGID 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;

CREATE SEQUENCE SECURPFC.SEQ_IDPERMIS 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;

CREATE SEQUENCE SECURPFC.SEQ_IDVISITA 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;

CREATE SEQUENCE SECURPFC.SEQ_IDREGISTRE 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;

CREATE SEQUENCE SECURPFC.SEQ_IDINCIDENCIA 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;
  
CREATE SEQUENCE SECURPFC.SEQ_IDLLIURAMENT 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 999999999
  NOCACHE
  CYCLE;

---------------------------
---- END  : CREATE SEQUENCE
---------------------------

----------------------------------------
---- BEGIN: CREATE TABLE AND CONSTRAINTS
----------------------------------------
CREATE TABLE SECURPFC.PAIS (
    nomPais VARCHAR2(60 CHAR),
    CONSTRAINT PK_PAIS PRIMARY KEY (nomPais)
); 

CREATE TABLE SECURPFC.CIUTAT (
    nomCiutat VARCHAR2(60 CHAR),
    nomPais   VARCHAR2(60 CHAR),
    CONSTRAINT PK_CIUTAT PRIMARY KEY (nomCiutat, nomPais)
); 
ALTER TABLE SECURPFC.CIUTAT
ADD (CONSTRAINT FK_CIUTAT_PAIS FOREIGN KEY (nomPais) REFERENCES SECURPFC.PAIS);

CREATE TABLE SECURPFC.EMPRESA (
    nomEmpresa VARCHAR2(50 CHAR),
    CONSTRAINT PK_EMPRESA PRIMARY KEY (nomEmpresa)
); 

CREATE TABLE SECURPFC.DELEGACIO (
    nomEmpresa   VARCHAR2(50 CHAR),
    nomDelegacio VARCHAR2(50 CHAR),
    adreca       VARCHAR2(100 CHAR) CONSTRAINT CDELEGACIO_adreca NOT NULL,
    codiPostal   VARCHAR2(9 CHAR)   CONSTRAINT CDELEGACIO_codiPostal NOT NULL,
    nomCiutat    VARCHAR2(60 CHAR)  CONSTRAINT CDELEGACIO_nomCiutat NOT NULL,
    nomPais      VARCHAR2(60 CHAR)  CONSTRAINT CDELEGACIO_nomPais NOT NULL,
    numTelefon   VARCHAR2(20 CHAR)  CONSTRAINT CDELEGACIO_numTelefon NOT NULL,
    CONSTRAINT PK_DELEGACIO PRIMARY KEY (nomEmpresa, nomDelegacio)
); 
ALTER TABLE SECURPFC.DELEGACIO
ADD (CONSTRAINT FK_DELEGACIO_EMPRESA FOREIGN KEY (nomEmpresa) REFERENCES SECURPFC.EMPRESA);
ALTER TABLE SECURPFC.DELEGACIO
ADD (CONSTRAINT FK_DELEGACIO_CIUTAT FOREIGN KEY (nomCiutat, nomPais) REFERENCES SECURPFC.CIUTAT);

CREATE TABLE SECURPFC.INCIDENCIA (
    idIncidencia    NUMBER,
    nomEmpresa      VARCHAR2(50 CHAR) CONSTRAINT CINCIDENCIA_nomEmpresa NOT NULL,
    nomDelegacio    VARCHAR2(50 CHAR) CONSTRAINT CINCIDENCIA_nomDelegacio NOT NULL,
    horaIncidencia  DATE              CONSTRAINT DINCIDENCIA_horaIncidencia NOT NULL,
    tipusIncidencia VARCHAR2(20 CHAR) CONSTRAINT CINCIDENCIA_tipusIncidencia NOT NULL,
    CONSTRAINT PK_INCIDENCIA PRIMARY KEY (idIncidencia)
);
ALTER TABLE SECURPFC.INCIDENCIA
ADD (CONSTRAINT FK_INCIDENCIA_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);

CREATE TABLE SECURPFC.DEPARTAMENT (
    nomDepartament VARCHAR2(30 CHAR),
    nomEmpresa     VARCHAR2(50 CHAR),
    nomDelegacio   VARCHAR2(50 CHAR),
    CONSTRAINT PK_DEPARTAMENT PRIMARY KEY (nomDepartament, nomEmpresa, nomDelegacio)
);
ALTER TABLE SECURPFC.DEPARTAMENT
ADD (CONSTRAINT FK_DEPARTAMENT_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);

CREATE TABLE SECURPFC.PERSONA (
    ID            VARCHAR2(20 CHAR),
    nom           VARCHAR2(20 CHAR) CONSTRAINT CPERSONA_nom NOT NULL,
    cognoms       VARCHAR2(40 CHAR) CONSTRAINT CPERSONA_cognoms NOT NULL,
    dataNaixement DATE              CONSTRAINT DPERSONA_dataNaixement NOT NULL,
    numTelefon    VARCHAR2(20 CHAR) CONSTRAINT CPERSONA_numTelefon NOT NULL,
    sexe          VARCHAR2(1 CHAR)  CONSTRAINT CPERSONA_sexe NOT NULL CONSTRAINT CKPERSONA_sexe CHECK(sexe IN ('H', 'M')),
    CONSTRAINT PK_PERSONA PRIMARY KEY (ID)
);

CREATE TABLE SECURPFC.EMPLEAT (
    IdEmpleat VARCHAR2(9 CHAR),
    ID        VARCHAR2(20 CHAR),
    nomDepartament VARCHAR2(30 CHAR) CONSTRAINT CEMPLEAT_nomDepartament NOT NULL,
    nomEmpresa     VARCHAR2(50 CHAR) CONSTRAINT CEMPLEAT_nomEmpresa NOT NULL,
    nomDelegacio   VARCHAR2(50 CHAR) CONSTRAINT CEMPLEAT_nomDelegacio NOT NULL,
    CONSTRAINT PK_EMPLEAT PRIMARY KEY (IdEmpleat, ID)
);
ALTER TABLE SECURPFC.EMPLEAT
ADD (CONSTRAINT FK_EMPLEAT_PERSONA FOREIGN KEY (ID) REFERENCES SECURPFC.PERSONA);
ALTER TABLE SECURPFC.EMPLEAT
ADD (CONSTRAINT FK_EMPLEAT_DEPARTAMENT FOREIGN KEY (nomDepartament, nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DEPARTAMENT);

CREATE TABLE SECURPFC.EXTERN (
    nomEmpresa VARCHAR2(50 CHAR),
    ID         VARCHAR2(20 CHAR),
    CONSTRAINT PK_EXTERN PRIMARY KEY (nomEmpresa, ID)
);
ALTER TABLE SECURPFC.EXTERN
ADD (CONSTRAINT FK_EXTERN_PERSONA FOREIGN KEY (ID) REFERENCES SECURPFC.PERSONA);

CREATE TABLE SECURPFC.VISITA (
    idVisita     NUMBER,
    nomEmpresa   VARCHAR2(50 CHAR) CONSTRAINT CVISITA_nomEmpresa NOT NULL,
    nomDelegacio VARCHAR2(50 CHAR) CONSTRAINT CVISITA_nomDelegacio NOT NULL,
    idEmpleat    VARCHAR2(9 CHAR)  CONSTRAINT CVISITA_idEmpleat NOT NULL,
    ID           VARCHAR2(20 CHAR) CONSTRAINT CVISITA_ID NOT NULL,
    horaEntrada  DATE              CONSTRAINT DVISITA_horaEntrada NOT NULL,
    horaSortida  DATE              CONSTRAINT DVISITA_horaSortida NOT NULL,
    motiuVisita  VARCHAR2(30 CHAR) CONSTRAINT CVISITA_motiuVisita NOT NULL,
    CONSTRAINT PK_VISITA PRIMARY KEY (idVisita)
);
ALTER TABLE SECURPFC.VISITA
ADD (CONSTRAINT FK_VISITA_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);
ALTER TABLE SECURPFC.VISITA
ADD (CONSTRAINT FK_VISITA_EMPLEAT FOREIGN KEY (idEmpleat, ID) REFERENCES SECURPFC.EMPLEAT);

CREATE TABLE SECURPFC.LLIURAMENT (
    idLliurament     NUMBER,
    nomEmpresa       VARCHAR2(50 CHAR) CONSTRAINT CLLIURAMENT_nomEmpresa NOT NULL,
    nomDelegacio     VARCHAR2(50 CHAR) CONSTRAINT CLLIURAMENT_nomDelegacio NOT NULL,
    IdEmpleat        VARCHAR2(9 CHAR)  CONSTRAINT CLLIURAMENT_IdEmpleat NOT NULL,
    ID               VARCHAR2(20 CHAR) CONSTRAINT CLLIURAMENT_ID NOT NULL,
    horaLliurament   DATE              CONSTRAINT DLLIURAMENT_horaLliurament NOT NULL,
    nomTransportista VARCHAR2(30 CHAR) CONSTRAINT CLLIURAMENT_nomTransportista NOT NULL,
    CONSTRAINT PK_LLIURAMENT PRIMARY KEY (idLliurament)
);
ALTER TABLE SECURPFC.LLIURAMENT
ADD (CONSTRAINT FK_LLIURAMENT_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);
ALTER TABLE SECURPFC.LLIURAMENT
ADD (CONSTRAINT FK_LLIURAMENT_EMPLEAT FOREIGN KEY (IdEmpleat, ID) REFERENCES SECURPFC.EMPLEAT);

CREATE TABLE SECURPFC.ZONA (
    idZona       VARCHAR2(30 CHAR),
    nomEmpresa   VARCHAR2(50 CHAR),
    nomDelegacio VARCHAR2(50 CHAR),
    CONSTRAINT PK_ZONA PRIMARY KEY (idZona, nomEmpresa, nomDelegacio)
);
ALTER TABLE SECURPFC.ZONA
ADD (CONSTRAINT FK_ZONA_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);

CREATE TABLE SECURPFC.REGISTRE (
    idRegistre   NUMBER,
    nomEmpresa   VARCHAR2(50 CHAR) CONSTRAINT CREGISTRE_nomEmpresa NOT NULL,
    nomDelegacio VARCHAR2(50 CHAR) CONSTRAINT CREGISTRE_nomDelegacio NOT NULL,   
    idZona       VARCHAR2(30 CHAR) CONSTRAINT CREGISTRE_idZona NOT NULL,
    ID           VARCHAR2(20 CHAR) CONSTRAINT CREGISTRE_ID NOT NULL,
    horaEntrada  DATE              CONSTRAINT DREGISTRE_horaEntrada NOT NULL,
    horaSortida  DATE              CONSTRAINT DREGISTRE_horaSortida NOT NULL, 
    CONSTRAINT PK_REGISTRE PRIMARY KEY (idRegistre)
);
ALTER TABLE SECURPFC.REGISTRE
ADD (CONSTRAINT FK_REGISTRE_DELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DELEGACIO);
ALTER TABLE SECURPFC.REGISTRE
ADD (CONSTRAINT FK_REGISTRE_ZONA FOREIGN KEY (idZona, nomEmpresa, nomDelegacio) REFERENCES SECURPFC.ZONA);
ALTER TABLE SECURPFC.REGISTRE
ADD (CONSTRAINT FK_REGISTRE_PERSONA FOREIGN KEY (ID) REFERENCES SECURPFC.PERSONA);

CREATE TABLE SECURPFC.PERMISACCES (
    idPermis       NUMBER,
    idZona         VARCHAR2(30 CHAR) CONSTRAINT CPERMISACCES_idZona NOT NULL,
    nomEmpresa     VARCHAR2(50 CHAR) CONSTRAINT CPERMISACCES_nomEmpresa NOT NULL,
    nomDelegacio   VARCHAR2(50 CHAR) CONSTRAINT CPERMISACCES_nomDelegacio NOT NULL,
    ID             VARCHAR2(20 CHAR),
    nomDepartament VARCHAR2(30 CHAR),
    horaInici      DATE              CONSTRAINT DPERMISACCES_horaInici NOT NULL,
    horaFi         DATE              CONSTRAINT DPERMISACCES_horaFi NOT NULL, 
    CONSTRAINT PK_PERMISACCES PRIMARY KEY (idPermis)
);
ALTER TABLE SECURPFC.PERMISACCES
ADD (CONSTRAINT FK_PERMISACCES_ZONA FOREIGN KEY (idZona, nomEmpresa, nomDelegacio) REFERENCES SECURPFC.ZONA);
ALTER TABLE SECURPFC.PERMISACCES
ADD (CONSTRAINT FK_PERMISACCES_PERSONA FOREIGN KEY (ID) REFERENCES SECURPFC.PERSONA);
ALTER TABLE SECURPFC.PERMISACCES
ADD (CONSTRAINT FK_PERMISACCES_DEPARTAMENT FOREIGN KEY (nomDepartament, nomEmpresa, nomDelegacio) REFERENCES SECURPFC.DEPARTAMENT);

CREATE TABLE SECURPFC.LOGBD (
    idLog         NUMBER,
    idProcediment VARCHAR2(40 CHAR) CONSTRAINT CLOGBD_idProcediment NOT NULL,
    dataExecucio  DATE              CONSTRAINT DLOGBD_dataExecucio NOT NULL,
    sortida       VARCHAR2(150 CHAR) CONSTRAINT CLOGBD_sortida NOT NULL,
    CONSTRAINT PK_LOGBD PRIMARY KEY (idLog)
);

----------------------------------------
---- END  : CREATE TABLE AND CONSTRAINTS
----------------------------------------
