-----------------------
---- BEGIN: CREATE USER
-----------------------

CREATE user SECURDW IDENTIFIED BY PFCBDREL_2015
DEFAULT TABLESPACE USERS
QUOTA UNLIMITED ON USERS
TEMPORARY TABLESPACE TEMP;

GRANT CREATE SESSION, CREATE TABLE, CREATE SEQUENCE, CREATE TYPE, CREATE PROCEDURE, ALTER SESSION TO SECURDW;
GRANT SELECT ON SECURPFC.REGISTRE TO SECURDW;
GRANT SELECT ON SECURPFC.VISITA TO SECURDW;
GRANT SELECT ON SECURPFC.EMPLEAT TO SECURDW;
GRANT SELECT ON SECURPFC.INCIDENCIA TO SECURDW;
GRANT SELECT ON SECURPFC.PERSONA TO SECURDW;
GRANT SELECT ON SECURPFC.DELEGACIO TO SECURDW;
GRANT SELECT ON SECURPFC.CIUTAT TO SECURDW;
GRANT SELECT ON SECURPFC.DEPARTAMENT TO SECURDW;
GRANT SELECT ON SECURPFC.SEQ_LOGID TO SECURDW;
GRANT INSERT ON SECURPFC.LOGBD TO SECURDW;

-----------------------
---- END  : CREATE USER
-----------------------

---------------------------
---- BEGIN: CREATE SEQUENCE
---------------------------

CREATE SEQUENCE SECURDW.SEQ_IDMOMENT 
  START WITH 1
  INCREMENT BY 1
  MAXVALUE 99999999999
  NOCACHE
  CYCLE;

---------------------------
---- END  : CREATE SEQUENCE
---------------------------

----------------------------------------
---- BEGIN: CREATE TABLE AND CONSTRAINTS
----------------------------------------

CREATE TABLE SECURDW.MDMOMENT(
  idMoment NUMBER,
  mSetmana  NUMBER CONSTRAINT NMDMOMENT_mSetmana NOT NULL,
  mMes      NUMBER CONSTRAINT NMDMOMENT_mMes NOT NULL,
  mAny      NUMBER CONSTRAINT NMDMOMENT_mAny NOT NULL,
  CONSTRAINT PK_MDMOMENT PRIMARY KEY (idMoment)
);

CREATE TABLE SECURDW.MDPERSONA (
  idPersona     VARCHAR2(20),
  dataNaixement DATE         CONSTRAINT DMDPERSONA_dataNaixement NOT NULL, 
  sexe          VARCHAR2(1)  CONSTRAINT CMDPERSONA_sexe NOT NULL,
  CONSTRAINT PK_MDPERSONA PRIMARY KEY (idPersona)
);

CREATE TABLE SECURDW.MDDELEGACIO (
    nomEmpresa   VARCHAR2(50), 
    nomDelegacio VARCHAR2(50),
    CONSTRAINT PK_MDDELEGACIO PRIMARY KEY (nomEmpresa, nomDelegacio)
);

CREATE TABLE SECURDW.MDREGISTRE (
  idRegistre     NUMBER,
  idMoment       NUMBER       CONSTRAINT NMDREGISTRE_idMoment NOT NULL,
  idPersona      VARCHAR2(20) CONSTRAINT CMDREGISTRE_idPersona NOT NULL,
  nomEmpresa     VARCHAR2(50) CONSTRAINT CMDREGISTRE_nomEmpresa NOT NULL,
  nomDelegacio   VARCHAR2(50) CONSTRAINT CMDREGISTRE_nomDelegacio NOT NULL,
  horaEntrada    DATE         CONSTRAINT DMDREGISTRE_horaEntrada NOT NULL,
  horaSortida    DATE         CONSTRAINT DMDREGISTRE_horaSortida NOT NULL,
  minutsRegistre NUMBER       CONSTRAINT NMDREGISTRE_minutsRegistre NOT NULL,
  CONSTRAINT PK_MDREGISTRE PRIMARY KEY (idRegistre)
);
ALTER TABLE SECURDW.MDREGISTRE
ADD (CONSTRAINT FK_MDREGISTRE_MDMOMENT FOREIGN KEY (idMoment) REFERENCES SECURDW.MDMOMENT);
ALTER TABLE SECURDW.MDREGISTRE
ADD (CONSTRAINT FK_MDREGISTRE_MDPERSONA FOREIGN KEY (idPersona) REFERENCES SECURDW.MDPERSONA);
ALTER TABLE SECURDW.MDREGISTRE
ADD (CONSTRAINT FK_MDREGISTRE_MDDELEGACIO FOREIGN KEY (nomEmpresa, nomDelegacio) REFERENCES SECURDW.MDDELEGACIO);

CREATE TABLE SECURDW.MDDEPARTAMENT (
    nomDepartament VARCHAR2(30),
    nomEmpresa     VARCHAR2(50), 
    nomDelegacio   VARCHAR2(50),
    CONSTRAINT PK_MDDEPARTAMENT PRIMARY KEY (nomDepartament, nomEmpresa, nomDelegacio)
);

CREATE TABLE SECURDW.MDVISITA (
  idVisita       NUMBER,
  idMoment       NUMBER       CONSTRAINT NMDVISITA_idMoment NOT NULL,
  nomDepartament VARCHAR2(30) CONSTRAINT CMDVISITA_nomDepartament NOT NULL,
  nomEmpresa     VARCHAR2(50) CONSTRAINT CMDVISITA_nomEmpresa NOT NULL,
  nomDelegacio   VARCHAR2(50) CONSTRAINT CMDVISITA_nomDelegacio NOT NULL,
  motiuVisita    VARCHAR2(30) CONSTRAINT CMDVISITA_motiuVisita NOT NULL,
  CONSTRAINT PK_MDVISITA PRIMARY KEY (idVisita)
);
ALTER TABLE SECURDW.MDVISITA
ADD (CONSTRAINT FK_MDVISITA_MDMOMENT FOREIGN KEY (idMoment) REFERENCES SECURDW.MDMOMENT);
ALTER TABLE SECURDW.MDVISITA
ADD (CONSTRAINT FK_MDVISITA_MDDEPARTAMENT FOREIGN KEY (nomDepartament, nomEmpresa, nomDelegacio) REFERENCES SECURDW.MDDEPARTAMENT);

CREATE TABLE SECURDW.MDCIUTAT (
    nomCiutat VARCHAR2(60),
    nomPais   VARCHAR2(60), 
    CONSTRAINT PK_MDCIUTAT PRIMARY KEY (nomCiutat, nomPais)
);

CREATE TABLE SECURDW.MDINCIDENCIA (
  idIncidencia    NUMBER,
  idMoment        NUMBER       CONSTRAINT NMDINCIDENCIA_idMoment NOT NULL,
  nomCiutat       VARCHAR2(60) CONSTRAINT CMDINCIDENCIA_nomCiutat NOT NULL,
  nomPais         VARCHAR2(60) CONSTRAINT CMDINCIDENCIA_nomPais NOT NULL, 
  tipusIncidencia VARCHAR2(20) CONSTRAINT CMDINCIDENCIA_tipusIncidencia NOT NULL,
  CONSTRAINT PK_MDINCIDENCIA PRIMARY KEY (idIncidencia)
);
ALTER TABLE SECURDW.MDINCIDENCIA
ADD (CONSTRAINT FK_MDINCIDENCIA_MDMOMENT FOREIGN KEY (idMoment) REFERENCES SECURDW.MDMOMENT);
ALTER TABLE SECURDW.MDINCIDENCIA
ADD (CONSTRAINT FK_MDINCIDENCIA_MDCIUTAT FOREIGN KEY (nomCiutat, nomPais) REFERENCES SECURDW.MDCIUTAT);

----------------------------------------
---- END  : CREATE TABLE AND CONSTRAINTS
----------------------------------------
