CREATE OR REPLACE PACKAGE Administrador AS
  PROCEDURE Pais_Inserir  (p_nomPais IN Pais.nomPais%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Pais_Esborrar (p_nomPais IN Pais.nomPais%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Pais_Llistat  (p_LlistatPaisos OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Ciutat_Inserir  (p_nomCiutat IN Ciutat.nomCiutat%TYPE, p_nomPais IN Ciutat.nomPais%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Ciutat_Esborrar (p_nomCiutat IN Ciutat.nomCiutat%TYPE, p_nomPais IN Ciutat.nomPais%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Ciutat_Llistat  (p_LlistatCiutats OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Empresa_Inserir  (p_nomEmpresa IN Empresa.nomEmpresa%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Empresa_Esborrar (p_nomEmpresa IN Empresa.nomEmpresa%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Empresa_Llistat  (p_LlistatEmpreses OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Delegacio_Inserir     (p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                                   p_adreca IN Delegacio.adreca%TYPE, p_codiPostal IN Delegacio.codiPostal%TYPE,
                                   p_nomCiutat IN Delegacio.nomCiutat%TYPE, p_nomPais IN Delegacio.nomPais%TYPE,
                                   p_numTelefon IN Delegacio.numTelefon%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Delegacio_Actualitzar (p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                                   p_adreca IN Delegacio.adreca%TYPE, p_codiPostal IN Delegacio.codiPostal%TYPE,
                                   p_nomCiutat IN Delegacio.nomCiutat%TYPE, p_nomPais IN Delegacio.nomPais%TYPE,
                                   p_numTelefon IN Delegacio.numTelefon%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Delegacio_Consulta    (p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                                   p_Delegacio OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Delegacio_Esborrar    (p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Delegacio_Llistat     (p_LlistatDelegacions OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Zona_Inserir  (p_idZona IN Zona.idZona%TYPE, p_nomEmpresa IN Zona.nomEmpresa%TYPE, p_nomDelegacio IN Zona.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Zona_Esborrar (p_idZona IN Zona.idZona%TYPE, p_nomEmpresa IN Zona.nomEmpresa%TYPE, p_nomDelegacio IN Zona.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Zona_Llistat  (p_LlistatZones OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Departament_Inserir  (p_nomDepartament IN Departament.nomDepartament%TYPE, 
                                  p_nomEmpresa IN Departament.nomEmpresa%TYPE, p_nomDelegacio IN Departament.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Departament_Esborrar (p_nomDepartament IN Departament.nomDepartament%TYPE, 
                                  p_nomEmpresa IN Departament.nomEmpresa%TYPE, p_nomDelegacio IN Departament.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Departament_Llistat  (p_LlistatDepartaments OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);                              
END Administrador;
/

CREATE OR REPLACE PACKAGE BODY Administrador AS
/****************************************
 * Nom procediment: Pais_Inserir
 * Descripci:      insereix un nou pas
 ****************************************/
PROCEDURE Pais_Inserir(p_nomPais IN Pais.nomPais%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Pais_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  INSERT INTO PAIS(nomPais) VALUES (p_nomPais);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: pais existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/****************************************
 * Nom procediment: Pais_Esborrar
 * Descripci:      elimina un pas
 ****************************************/
PROCEDURE Pais_Esborrar(p_nomPais IN Pais.nomPais%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  PAIS_NO_TROBAT EXCEPTION;
  BEGIN
  vProcedureName := 'Pais_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM PAIS WHERE nomPais = p_nomPais;
  IF (SQL%NOTFOUND) THEN
     RAISE PAIS_NO_TROBAT;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PAIS_NO_TROBAT THEN
    p_sortida:='ERROR: pais no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*********************************************
 * Nom procediment: Pais_Llistat
 * Descripci:      obt el llistat de pasos
 *********************************************/
PROCEDURE Pais_Llistat(p_LlistatPaisos OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Pais_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatPaisos FOR
      SELECT * FROM PAIS
      ORDER BY nomPais;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/****************************************
 * Nom procediment: Ciutat_Inserir
 * Descripci:      insereix una ciutat
 ****************************************/
PROCEDURE Ciutat_Inserir(p_nomCiutat IN Ciutat.nomCiutat%TYPE, p_nomPais IN Ciutat.nomPais%TYPE, p_sortida OUT VARCHAR2) IS
  cntPais   NUMBER;
  PAIS_NO_TROBAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Ciutat_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntPais := 0;
  SELECT COUNT(1) INTO cntPais FROM PAIS where nomPais = p_nomPais;
  IF (cntPais = 0) THEN
    RAISE PAIS_NO_TROBAT;
  END IF;                                                                                                            
  INSERT INTO CIUTAT(nomCiutat, nomPais) VALUES (p_nomCiutat, p_nomPais);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PAIS_NO_TROBAT THEN
    p_sortida:='ERROR: pais no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: ciutat existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/****************************************
 * Nom procediment: Ciutat_Esborrar
 * Descripci:      elimina una ciutat
 ****************************************/
PROCEDURE Ciutat_Esborrar(p_nomCiutat IN Ciutat.nomCiutat%TYPE, p_nomPais IN Ciutat.nomPais%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  CIUTAT_NO_TROBADA EXCEPTION;
  BEGIN
  vProcedureName := 'Ciutat_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM CIUTAT WHERE nomCiutat = p_nomCiutat AND nomPais = p_nomPais;
  IF (SQL%NOTFOUND) THEN
     RAISE CIUTAT_NO_TROBADA;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN CIUTAT_NO_TROBADA THEN
    p_sortida:='ERROR: ciutat no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**********************************************
 * Nom procediment: Ciutat_Llistat
 * Descripci:      obt el llistat de ciutats
 **********************************************/
PROCEDURE Ciutat_Llistat(p_LlistatCiutats OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Ciutat_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatCiutats FOR
      SELECT * FROM CIUTAT
      ORDER BY nomPais, nomCiutat;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*********************************************
 * Nom procediment: Empresa_Inserir
 * Descripci:      insereix una nova empresa
 *********************************************/
PROCEDURE Empresa_Inserir(p_nomEmpresa IN Empresa.nomEmpresa%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empresa_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  INSERT INTO EMPRESA(nomEmpresa) VALUES (p_nomEmpresa);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: empresa existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/****************************************
 * Nom procediment: Empresa_Esborrar
 * Descripci:      elimina una empresa
 ****************************************/
PROCEDURE Empresa_Esborrar(p_nomEmpresa IN Empresa.nomEmpresa%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  no_data_found EXCEPTION;
  BEGIN
  vProcedureName := 'Empresa_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM EMPRESA WHERE nomEmpresa = p_nomEmpresa;
  IF (SQL%NOTFOUND) THEN
     RAISE no_data_found;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN no_data_found THEN
    p_sortida:='ERROR: empresa no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**********************************************
 * Nom procediment: Empresa_Llistat
 * Descripci:      obt el llistat d'empreses
 **********************************************/
PROCEDURE Empresa_Llistat(p_LlistatEmpreses OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empresa_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatEmpreses FOR
      SELECT * FROM EMPRESA
      ORDER BY nomEmpresa;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************
 * Nom procediment: Delegacio_Inserir
 * Descripci:      insereix una delegaci
 ******************************************/
PROCEDURE Delegacio_Inserir(p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                             p_adreca IN Delegacio.adreca%TYPE, p_codiPostal IN Delegacio.codiPostal%TYPE,
                             p_nomCiutat IN Delegacio.nomCiutat%TYPE, p_nomPais IN Delegacio.nomPais%TYPE,
                             p_numTelefon IN Delegacio.numTelefon%TYPE, p_sortida OUT VARCHAR2) IS
  cntEmpresa NUMBER;
  cntCiutat   NUMBER;
  EMPRESA_NO_TROBADA EXCEPTION;
  CIUTAT_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Delegacio_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntEmpresa := 0;
  cntCiutat := 0;
  SELECT COUNT(1) INTO cntEmpresa FROM EMPRESA where nomEmpresa = p_nomEmpresa;
  IF (cntEmpresa = 0) THEN
    RAISE EMPRESA_NO_TROBADA;
  END IF;                                                                                                            
  SELECT COUNT(1) INTO cntCiutat FROM CIUTAT where nomCiutat = p_nomCiutat AND nomPais = p_nomPais;
  IF (cntCiutat = 0) THEN
    RAISE CIUTAT_NO_TROBADA;
  END IF;                                                                                                            
  INSERT INTO DELEGACIO(nomEmpresa, nomDelegacio, adreca, codiPostal, nomCiutat, nomPais, numTelefon) 
  VALUES (p_nomEmpresa, p_nomDelegacio, p_adreca, p_codiPostal, p_nomCiutat, p_nomPais, p_numTelefon);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EMPRESA_NO_TROBADA THEN
    p_sortida:='ERROR: empresa no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN CIUTAT_NO_TROBADA THEN
    p_sortida:='ERROR: ciutat no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
           
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: delegacio existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***************************************************************************
 * Nom procediment: Delegacio_Actualitzar
 * Descripci:      actualitza dades delegacio a partir de la clau primria
 ***************************************************************************/
PROCEDURE Delegacio_Actualitzar(p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                                p_adreca IN Delegacio.adreca%TYPE, p_codiPostal IN Delegacio.codiPostal%TYPE,
                                p_nomCiutat IN Delegacio.nomCiutat%TYPE, p_nomPais IN Delegacio.nomPais%TYPE,
                                p_numTelefon IN Delegacio.numTelefon%TYPE, p_sortida OUT VARCHAR2) IS
  cntEmpresa   NUMBER;
  cntCiutat    NUMBER;
  cntDelegacio NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  EMPRESA_NO_TROBADA EXCEPTION;
  CIUTAT_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Delegacio_Actualitzar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntEmpresa := 0;
  cntCiutat := 0;
  SELECT COUNT(1) INTO cntEmpresa FROM EMPRESA where nomEmpresa = p_nomEmpresa;
  IF (cntEmpresa = 0) THEN
    RAISE EMPRESA_NO_TROBADA;
  END IF;                                                                                                            
  SELECT COUNT(1) INTO cntCiutat FROM CIUTAT where nomCiutat = p_nomCiutat AND nomPais = p_nomPais;
  IF (cntCiutat = 0) THEN
    RAISE CIUTAT_NO_TROBADA;
  END IF;                                 
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;
  
  UPDATE DELEGACIO SET adreca = p_adreca, codiPostal = p_codiPostal, nomCiutat = p_nomCiutat, 
  nomPais = p_nomPais, numTelefon = p_numTelefon WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EMPRESA_NO_TROBADA THEN
    p_sortida:='ERROR: empresa no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN CIUTAT_NO_TROBADA THEN
    p_sortida:='ERROR: ciutat no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
           
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/********************************************
 * Nom procediment: Delegacio_Consulta
 * Descripci:      consulta dades delegaci
 ********************************************/
PROCEDURE Delegacio_Consulta (p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                              p_Delegacio OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  cntDelegacio NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Delegacio_Consulta';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntDelegacio := 0;
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;                                                                                                            
    
  OPEN p_Delegacio FOR
      SELECT * FROM DELEGACIO
      WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*****************************************
 * Nom procediment: Delegacio_Esborrar
 * Descripci:      elimina una delegaci
 *****************************************/
PROCEDURE Delegacio_Esborrar(p_nomEmpresa IN Delegacio.nomEmpresa%TYPE, p_nomDelegacio IN Delegacio.nomDelegacio%TYPE,  
                             p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  DELEGACIO_NO_TROBADA EXCEPTION;
  BEGIN
  vProcedureName := 'Delegacio_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM DELEGACIO WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (SQL%NOTFOUND) THEN
     RAISE DELEGACIO_NO_TROBADA;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**************************************************
 * Nom procediment: Delegacio_Llistat
 * Descripci:      obt el llistat de delegacions
 **************************************************/
PROCEDURE Delegacio_Llistat(p_LlistatDelegacions OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Delegacio_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatDelegacions FOR
      SELECT * FROM DELEGACIO
      ORDER BY nomEmpresa, nomDelegacio;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*******************************************************
 * Nom procediment: Zona_Inserir
 * Descripci:      crea una nova zona en una delegaci
 *******************************************************/
PROCEDURE Zona_Inserir(p_idZona IN Zona.idZona%TYPE, p_nomEmpresa IN Zona.nomEmpresa%TYPE, p_nomDelegacio IN Zona.nomDelegacio%TYPE,
                       p_sortida OUT VARCHAR2) IS
  cntDelegacio NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Zona_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntDelegacio := 0;
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO WHERE nomEmpresa = p_nomEmpresa
  AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;                                                                                                            
  INSERT INTO ZONA(idZona, nomEmpresa, nomDelegacio) VALUES (p_idZona, p_nomEmpresa, p_nomDelegacio);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: zona existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/****************************************************
 * Nom procediment: Zona_Esborrar
 * Descripci:      elimina una zona d'una delegaci
 ****************************************************/
PROCEDURE Zona_Esborrar(p_idZona IN Zona.idZona%TYPE, p_nomEmpresa IN Zona.nomEmpresa%TYPE, p_nomDelegacio IN Zona.nomDelegacio%TYPE,
                        p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  ZONA_NO_TROBADA EXCEPTION;
  BEGIN
  vProcedureName := 'Zona_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM ZONA WHERE idZona = p_idZona AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (SQL%NOTFOUND) THEN
     RAISE ZONA_NO_TROBADA;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN ZONA_NO_TROBADA THEN
    p_sortida:='ERROR: zona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/********************************************
 * Nom procediment: Zona_Llistat
 * Descripci:      obt el llistat de zones
 ********************************************/
PROCEDURE Zona_Llistat(p_LlistatZones OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Zona_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatZones FOR
      SELECT * FROM ZONA
      ORDER BY nomEmpresa, nomDelegacio, idZona;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***********************************************
 * Nom procediment: Departament_Inserir
 * Descripci:      insereix un nou departament
 ***********************************************/
PROCEDURE Departament_Inserir(p_nomDepartament IN Departament.nomDepartament%TYPE, 
                              p_nomEmpresa IN Departament.nomEmpresa%TYPE, p_nomDelegacio IN Departament.nomDelegacio%TYPE,  
                              p_sortida OUT VARCHAR2) IS
  cntDelegacio NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Departament_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntDelegacio := 0;
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO where nomEmpresa = p_nomEmpresa
  AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;                                                                                                            
  INSERT INTO DEPARTAMENT(nomDepartament, nomEmpresa, nomDelegacio) 
  VALUES (p_nomDepartament, p_nomEmpresa, p_nomDelegacio);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: departament existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************
 * Nom procediment: Departament_Esborrar
 * Descripci:      elimina un departament
 ******************************************/
PROCEDURE Departament_Esborrar(p_nomDepartament IN Departament.nomDepartament%TYPE, 
                               p_nomEmpresa IN Departament.nomEmpresa%TYPE, p_nomDelegacio IN Departament.nomDelegacio%TYPE,  
                               p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  DEPARTAMENT_NO_TROBAT EXCEPTION;
  BEGIN
  vProcedureName := 'Departament_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM DEPARTAMENT WHERE nomDepartament = p_nomDepartament AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (SQL%NOTFOUND) THEN
     RAISE DEPARTAMENT_NO_TROBAT;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DEPARTAMENT_NO_TROBAT THEN
    p_sortida:='ERROR: departament no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***************************************************
 * Nom procediment: Departament_Llistat
 * Descripci:      obt el llistat de departaments
 ***************************************************/
PROCEDURE Departament_Llistat(p_LlistatDepartaments OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Departament_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatDepartaments FOR
      SELECT * FROM DEPARTAMENT
      ORDER BY nomEmpresa, nomDelegacio, nomDepartament;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

END Administrador;
/
