CREATE OR REPLACE PACKAGE Personal AS
  PROCEDURE Persona_Inserir     (p_ID IN Persona.ID%TYPE, p_nom IN Persona.nom%TYPE, p_cognoms IN Persona.cognoms%TYPE,
                                 p_dataNaixement IN Persona.dataNaixement%TYPE, p_numTelefon IN Persona.numTelefon%TYPE,
                                 p_sexe IN Persona.sexe%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Persona_Actualitzar (p_ID IN Persona.ID%TYPE, p_nom IN Persona.nom%TYPE, p_cognoms IN Persona.cognoms%TYPE,
                                 p_dataNaixement IN Persona.dataNaixement%TYPE, p_numTelefon IN Persona.numTelefon%TYPE,
                                 p_sexe IN Persona.sexe%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Persona_Consulta    (p_ID IN Persona.ID%TYPE, p_Persona OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Persona_Esborrar    (p_ID IN Persona.ID%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Persona_Llistat     (p_LlistatPersones OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Empleat_Inserir     (p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, 
                                 p_nomDepartament IN Empleat.nomDepartament%TYPE, p_nomEmpresa IN Empleat.nomEmpresa%TYPE, 
                                 p_nomDelegacio IN Empleat.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Empleat_Actualitzar (p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, 
                                 p_nomDepartament IN Empleat.nomDepartament%TYPE, p_nomEmpresa IN Empleat.nomEmpresa%TYPE, 
                                 p_nomDelegacio IN Empleat.nomDelegacio%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Empleat_Consulta    (p_ID IN Persona.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, p_Empleat OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Empleat_Esborrar    (p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Empleat_Llistat     (p_LlistatEmpleats OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
  PROCEDURE Extern_Inserir   (p_ID IN Extern.ID%TYPE, p_nomEmpresa IN Extern.nomEmpresa%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Extern_Esborrar  (p_ID IN Extern.ID%TYPE, p_nomEmpresa IN Extern.nomEmpresa%TYPE, p_sortida OUT VARCHAR2);  
  PROCEDURE Extern_Llistat   (p_LlistatExterns OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
END Personal;
/

CREATE OR REPLACE PACKAGE BODY Personal AS
/*****************************************
 * Nom procediment: Persona_Inserir
 * Descripci:      crea una nova persona
 *****************************************/
PROCEDURE Persona_Inserir(p_ID IN Persona.ID%TYPE, p_nom IN Persona.nom%TYPE, p_cognoms IN Persona.cognoms%TYPE,
                          p_dataNaixement IN Persona.dataNaixement%TYPE, p_numTelefon IN Persona.numTelefon%TYPE,
                          p_sexe IN Persona.sexe%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Persona_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  INSERT INTO PERSONA(ID, nom, cognoms, dataNaixement, numTelefon, sexe) 
  VALUES (p_ID, p_nom, p_cognoms, p_dataNaixement, p_numTelefon, p_sexe);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: persona existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/********************************************
 * Nom procediment: Persona_Actualitzar
 * Descripci:      actualitza dades persona
 ********************************************/
PROCEDURE Persona_Actualitzar(p_ID IN Persona.ID%TYPE, p_nom IN Persona.nom%TYPE, p_cognoms IN Persona.cognoms%TYPE,
                              p_dataNaixement IN Persona.dataNaixement%TYPE, p_numTelefon IN Persona.numTelefon%TYPE,
                              p_sexe IN Persona.sexe%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  cntPersona NUMBER;
  PERSONA_NO_TROBADA EXCEPTION;
  
  BEGIN
  vProcedureName := 'Persona_Actualitzar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  SELECT COUNT(1) INTO cntPersona FROM PERSONA WHERE ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;
  
  UPDATE PERSONA SET nom = p_nom, cognoms = p_cognoms, dataNaixement = p_dataNaixement, numTelefon = p_numTelefon, sexe = p_sexe 
  WHERE ID = p_ID;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************
 * Nom procediment: Persona_Consulta
 * Descripci:      consulta dades persona
 ******************************************/
PROCEDURE Persona_Consulta (p_ID IN Persona.ID%TYPE, p_Persona OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  cntPersona NUMBER;
  PERSONA_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Persona_Consulta';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  SELECT COUNT(1) INTO cntPersona FROM PERSONA WHERE ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;                                                                                                            
  
  OPEN p_Persona FOR
      SELECT * FROM PERSONA WHERE ID = p_ID;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*************************************
 * Nom procediment: Persona_Esborrar
 * Descripci:      elimina persona
 *************************************/
PROCEDURE Persona_Esborrar(p_ID IN Persona.ID%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  PERSONA_NO_TROBADA EXCEPTION;
  BEGIN
  vProcedureName := 'Persona_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM PERSONA WHERE ID = p_ID;
  IF (SQL%NOTFOUND) THEN
     RAISE PERSONA_NO_TROBADA;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***********************************************
 * Nom procediment: Persona_Llistat
 * Descripci:      obt el llistat de persones
 ***********************************************/
PROCEDURE Persona_Llistat (p_LlistatPersones OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Persona_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatPersones FOR
      SELECT * FROM PERSONA;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;
                              
/*******************************************
 * Nom procediment: Empleat_Inserir
 * Descripci:      insereix un nou empleat 
 *******************************************/
PROCEDURE Empleat_Inserir(p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, 
                          p_nomDepartament IN Empleat.nomDepartament%TYPE, p_nomEmpresa IN Empleat.nomEmpresa%TYPE, 
                          p_nomDelegacio IN Empleat.nomDelegacio%TYPE,p_sortida OUT VARCHAR2) IS
  cntPersona NUMBER;
  cntDepartament NUMBER;
  PERSONA_NO_TROBADA EXCEPTION;
  DEPARTAMENT_NO_TROBAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empleat_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntPersona := 0;
  SELECT COUNT(1) INTO cntPersona FROM PERSONA where ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;                                                                                                            
  cntDepartament := 0;
  SELECT COUNT(1) INTO cntDepartament FROM DEPARTAMENT where nomDepartament = p_nomDepartament AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDepartament = 0) THEN
    RAISE DEPARTAMENT_NO_TROBAT;
  END IF;                                                                                                            
  INSERT INTO EMPLEAT(ID, idEmpleat, nomDepartament, nomEmpresa, nomDelegacio) 
  VALUES (p_ID, p_idEmpleat, p_nomDepartament, p_nomEmpresa, p_nomDelegacio);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DEPARTAMENT_NO_TROBAT THEN
    p_sortida:='ERROR: departament no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: empleat existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/********************************************
 * Nom procediment: Empleat_Actualitzar
 * Descripci:      actualitza dades empleat
 ********************************************/
PROCEDURE Empleat_Actualitzar(p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, 
                              p_nomDepartament IN Empleat.nomDepartament%TYPE, p_nomEmpresa IN Empleat.nomEmpresa%TYPE, 
                              p_nomDelegacio IN Empleat.nomDelegacio%TYPE,p_sortida OUT VARCHAR2) IS
  cntEmpleat NUMBER;
  cntDepartament NUMBER;
  EMPLEAT_NO_TROBAT EXCEPTION;
  DEPARTAMENT_NO_TROBAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empleat_Actualitzar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntEmpleat := 0;
  SELECT COUNT(1) INTO cntEmpleat FROM EMPLEAT WHERE ID = p_ID AND idEmpleat = p_idEmpleat;
  IF (cntEmpleat = 0) THEN
    RAISE EMPLEAT_NO_TROBAT;
  END IF;                                                                                                            
  cntDepartament := 0;
  SELECT COUNT(1) INTO cntDepartament FROM DEPARTAMENT where nomDepartament = p_nomDepartament AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDepartament = 0) THEN
    RAISE DEPARTAMENT_NO_TROBAT;
  END IF;                                                                                                            
  UPDATE EMPLEAT SET nomDepartament = p_nomDepartament, nomEmpresa = p_nomEmpresa, nomDelegacio = p_nomDelegacio 
  WHERE ID = p_ID AND idEmpleat = p_idEmpleat;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EMPLEAT_NO_TROBAT THEN
    p_sortida:='ERROR: empleat no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));

  WHEN DEPARTAMENT_NO_TROBAT THEN
    p_sortida:='ERROR: departament no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************
 * Nom procediment: Empleat_Consulta
 * Descripci:      consulta dades empleat
 ******************************************/
PROCEDURE Empleat_Consulta (p_ID IN Persona.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, p_Empleat OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  cntEmpleat NUMBER;
  EMPLEAT_NO_TROBAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empleat_Consulta';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  SELECT COUNT(1) INTO cntEmpleat FROM EMPLEAT WHERE ID = p_ID AND idEmpleat = p_idEmpleat;
  IF (cntEmpleat = 0) THEN
    RAISE EMPLEAT_NO_TROBAT;
  END IF;                                                                                                            
 
  OPEN p_Empleat FOR
      SELECT * FROM EMPLEAT WHERE ID = p_ID AND idEmpleat = p_idEmpleat;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EMPLEAT_NO_TROBAT THEN
    p_sortida:='ERROR: empleat no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/************************************
 * Nom procediment: Empleat_Esborrar
 * Descripci:      elimina empleat
 ************************************/
PROCEDURE Empleat_Esborrar(p_ID IN Empleat.ID%TYPE, p_idEmpleat IN Empleat.idEmpleat%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  EMPLEAT_NO_TROBAT EXCEPTION;
  BEGIN
  vProcedureName := 'Empleat_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM EMPLEAT WHERE ID = p_ID AND idEmpleat = p_idEmpleat;
  IF (SQL%NOTFOUND) THEN
     RAISE EMPLEAT_NO_TROBAT;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EMPLEAT_NO_TROBAT THEN
    p_sortida:='ERROR: empleat no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**********************************************
 * Nom procediment: Empleat_Llistat
 * Descripci:      obt el llistat d'empleats
 **********************************************/
PROCEDURE Empleat_Llistat (p_LlistatEmpleats OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Empleat_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatEmpleats FOR
      SELECT * FROM EMPLEAT;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************
 * Nom procediment: Extern_Inserir
 * Descripci:      insereix un nou extern 
 ******************************************/
PROCEDURE Extern_Inserir(p_ID IN Extern.ID%TYPE, p_nomEmpresa IN Extern.nomEmpresa%TYPE, p_sortida OUT VARCHAR2) IS
  cntPersona NUMBER;
  PERSONA_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Extern_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntPersona := 0;
  SELECT COUNT(1) INTO cntPersona FROM PERSONA where ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;                                                                                                            
  INSERT INTO EXTERN(ID, nomEmpresa) VALUES (p_ID, p_nomEmpresa);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: extern existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**************************************
 * Nom procediment: Extern_Esborrar
 * Descripci:      elimina un extern
 **************************************/
PROCEDURE Extern_Esborrar(p_ID IN Extern.ID%TYPE, p_nomEmpresa IN Extern.nomEmpresa%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  EXTERN_NO_TROBAT EXCEPTION;
  BEGIN
  vProcedureName := 'Extern_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM EXTERN WHERE ID = p_ID AND nomEmpresa = p_nomEmpresa;
  IF (SQL%NOTFOUND) THEN
     RAISE EXTERN_NO_TROBAT;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN EXTERN_NO_TROBAT THEN
    p_sortida:='ERROR: extern no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*********************************************
 * Nom procediment: Extern_Llistat
 * Descripci:      obt el llistat d'externs
 *********************************************/
PROCEDURE Extern_Llistat (p_LlistatExterns OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Extern_Llistat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  OPEN p_LlistatExterns FOR
      SELECT * FROM EXTERN;

  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

END Personal;
/
