CREATE OR REPLACE PACKAGE Usuari AS
  PROCEDURE Registre_Inserir          (p_idRegistre OUT Registre.idRegistre%TYPE, p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                                       p_idZona IN Registre.idZona%TYPE, p_ID IN Registre.ID%TYPE, p_horaEntrada IN Registre.horaEntrada%TYPE, 
                                       p_horaSortida IN Registre.horaSortida%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Registre_ActSortida       (p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                                       p_idZona IN Registre.idZona%TYPE, p_ID IN Registre.ID%TYPE, p_horaSortida IN Registre.horaSortida%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Registre_Esborrar         (p_idRegistre IN Registre.idRegistre%TYPE, p_sortida OUT VARCHAR2);
  PROCEDURE Registre_LlistatZona      (p_idZona IN Registre.idZona%TYPE, p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE, 
                                       p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, p_LlistatRegistreZona OUT SYS_REFCURSOR, 
                                       p_sortida OUT VARCHAR2);
  PROCEDURE Registre_LlistatDelegacio (p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                                       p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, p_LlistatRegistreDelegacio OUT SYS_REFCURSOR, 
                                       p_sortida OUT VARCHAR2);
  PROCEDURE Registre_LlistatPersona   (p_ID IN Registre.ID%TYPE, p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, 
                                       p_LlistatRegistrePersona OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
END Usuari;
/

CREATE OR REPLACE PACKAGE BODY Usuari AS
/********************************************
 * Nom procediment: Registre_Inserir
 * Descripci:      insereix un nou registre 
 ********************************************/
PROCEDURE Registre_Inserir(p_idRegistre OUT Registre.idRegistre%TYPE, p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                           p_idZona IN Registre.idZona%TYPE, p_ID IN Registre.ID%TYPE, p_horaEntrada IN Registre.horaEntrada%TYPE, 
                           p_horaSortida IN Registre.horaSortida%TYPE, p_sortida OUT VARCHAR2) IS
  cntDelegacio NUMBER;
  cntZona      NUMBER;
  cntPersona   NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  ZONA_NO_TROBADA EXCEPTION;
  PERSONA_NO_TROBADA EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Registre_Inserir';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntDelegacio := 0;
  cntZona := 0;
  cntPersona := 0;
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO where nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;                              
  SELECT COUNT(1) INTO cntZona FROM ZONA where idZona = p_idZona AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntZona = 0) THEN
    RAISE ZONA_NO_TROBADA;
  END IF;                              
  SELECT COUNT(1) INTO cntPersona FROM PERSONA where ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;      
  p_idRegistre := SEQ_IDREGISTRE.nextval;
  INSERT INTO REGISTRE(idRegistre, nomEmpresa, nomDelegacio, idZona, ID, horaEntrada, horaSortida) 
  VALUES (p_idRegistre, p_nomEmpresa, p_nomDelegacio, p_idZona, p_ID, p_horaEntrada, p_horaSortida);
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN ZONA_NO_TROBADA THEN
    p_sortida:='ERROR: zona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: registre existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en inserir - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/********************************************************
 * Nom procediment: Registre_ActSortida
 * Descripci:      actualitza hora sortida del registre
 ********************************************************/
PROCEDURE Registre_ActSortida(p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                              p_idZona IN Registre.idZona%TYPE, p_ID IN Registre.ID%TYPE, p_horaSortida IN Registre.horaSortida%TYPE, p_sortida OUT VARCHAR2) IS
  cntDelegacio     NUMBER;
  cntZona          NUMBER;
  cntPersona       NUMBER;
  cntRegistre      NUMBER;
  cntRegistreNoAct NUMBER;
  DELEGACIO_NO_TROBADA EXCEPTION;
  ZONA_NO_TROBADA EXCEPTION;
  PERSONA_NO_TROBADA EXCEPTION;
  REGISTRE_NO_TROBAT EXCEPTION;
  REGISTRE_ACTUALITZAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  BEGIN
  vProcedureName := 'Registre_ActSortida';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  cntDelegacio := 0;
  cntZona := 0;
  cntPersona := 0;
  cntRegistre:= 0;
  cntRegistreNoAct := 0;
  SELECT COUNT(1) INTO cntDelegacio FROM DELEGACIO where nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_TROBADA;
  END IF;                              
  SELECT COUNT(1) INTO cntZona FROM ZONA where idZona = p_idZona AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntZona = 0) THEN
    RAISE ZONA_NO_TROBADA;
  END IF;                              
  SELECT COUNT(1) INTO cntPersona FROM PERSONA where ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_TROBADA;
  END IF;                   
  SELECT COUNT(1) INTO cntRegistre FROM REGISTRE WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio AND
  idZona = p_idZona AND ID = p_ID;
  IF (cntRegistre = 0) THEN
    RAISE REGISTRE_NO_TROBAT;
  END IF;                   
  SELECT COUNT(1) INTO cntRegistreNoAct FROM REGISTRE WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio AND
  idZona = p_idZona AND ID = p_ID AND horaEntrada = horaSortida;
  IF (cntRegistreNoAct = 0) THEN
    RAISE REGISTRE_ACTUALITZAT;
  END IF;
  
  UPDATE REGISTRE SET horaSortida = p_horaSortida WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio AND
  idZona = p_idZona AND ID = p_ID AND horaEntrada = horaSortida;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_TROBADA THEN
    p_sortida:='ERROR: delegacio no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN ZONA_NO_TROBADA THEN
    p_sortida:='ERROR: zona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN PERSONA_NO_TROBADA THEN
    p_sortida:='ERROR: persona no trobada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN REGISTRE_NO_TROBAT THEN
    p_sortida:='ERROR: registre no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN REGISTRE_ACTUALITZAT THEN
    p_sortida:='ERROR: registre ja actualitzat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en actualitzar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***************************************
 * Nom procediment: Registre_Esborrar
 * Descripci:      elimina un registre
 ***************************************/
PROCEDURE Registre_Esborrar(p_idRegistre IN Registre.idRegistre%TYPE, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  REGISTRE_NO_TROBAT EXCEPTION;
  BEGIN
  vProcedureName := 'Registre_Esborrar';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  DELETE FROM REGISTRE WHERE idRegistre = p_idRegistre;
  IF (SQL%NOTFOUND) THEN
     RAISE REGISTRE_NO_TROBAT;
  END IF;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN REGISTRE_NO_TROBAT THEN
    p_sortida:='ERROR: registre no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en esborrar - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*****************************************************************************
 * Nom procediment: Registre_LlistatZona
 * Descripci:      obt el llistat de registres per zona
 *                  entre els moments inicial i final passats com a parmetre
 *****************************************************************************/
PROCEDURE Registre_LlistatZona (p_idZona IN Registre.idZona%TYPE, p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                                p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, p_LlistatRegistreZona OUT SYS_REFCURSOR, 
                                p_sortida OUT VARCHAR2) IS  
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  cntZona NUMBER;
  ZONA_NO_REGISTRADA EXCEPTION;
  BEGIN
  vProcedureName := 'Registre_LlistatZona';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  SELECT COUNT(1) INTO cntZona FROM REGISTRE where idZona = p_idZona AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntZona = 0) THEN
    RAISE ZONA_NO_REGISTRADA;
  END IF;
  
  OPEN p_LlistatRegistreZona FOR
      SELECT * FROM REGISTRE WHERE idZona = p_idZona AND nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio AND 
      horaEntrada >= p_momentInici AND horaSortida <= p_momentFi
      ORDER BY horaEntrada, ID;
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN ZONA_NO_REGISTRADA THEN
    p_sortida:='ERROR: zona no registrada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/*****************************************************************************
 * Nom procediment: Registre_LlistatDelegacio
 * Descripci:      obt el llistat de registres per delegaci
 *                  entre els moments inicial i final passats com a parmetre
 *****************************************************************************/
PROCEDURE Registre_LlistatDelegacio (p_nomEmpresa IN Registre.nomEmpresa%TYPE, p_nomDelegacio IN Registre.nomDelegacio%TYPE,
                                     p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, p_LlistatRegistreDelegacio OUT SYS_REFCURSOR, 
                                     p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  cntDelegacio NUMBER;
  DELEGACIO_NO_REGISTRADA EXCEPTION;
  BEGIN
  vProcedureName := 'Registre_LlistatDelegacio';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  SELECT COUNT(1) INTO cntDelegacio FROM REGISTRE WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio;
  IF (cntDelegacio = 0) THEN
    RAISE DELEGACIO_NO_REGISTRADA;
  END IF;
  
  OPEN p_LlistatRegistreDelegacio FOR
      SELECT * FROM REGISTRE WHERE nomEmpresa = p_nomEmpresa AND nomDelegacio = p_nomDelegacio AND 
      horaEntrada >= p_momentInici AND horaSortida <= p_momentFi
      ORDER BY horaEntrada, ID;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DELEGACIO_NO_REGISTRADA THEN
    p_sortida:='ERROR: delegacio no registrada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;
                                     
/*****************************************************************************
 * Nom procediment: Registre_LlistatPersona
 * Descripci:      obt el llistat de registres per persona
 *                  entre els moments inicial i final passats com a parmetre
 *****************************************************************************/
PROCEDURE Registre_LlistatPersona (p_ID IN Registre.ID%TYPE, p_momentInici IN Registre.horaEntrada%TYPE, p_momentFi IN Registre.horaEntrada%TYPE, 
                                   p_LlistatRegistrePersona OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  cntPersona NUMBER;
  PERSONA_NO_REGISTRADA EXCEPTION;
  BEGIN
  vProcedureName := 'Registre_LlistatPersona';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
  
  SELECT COUNT(1) INTO cntPersona FROM REGISTRE WHERE ID = p_ID;
  IF (cntPersona = 0) THEN
    RAISE PERSONA_NO_REGISTRADA;
  END IF;
  
  OPEN p_LlistatRegistrePersona FOR
      SELECT * FROM REGISTRE WHERE ID = p_ID AND horaEntrada >= p_momentInici AND horaSortida <= p_momentFi
      ORDER BY horaEntrada, ID;
  
  INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
         VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN PERSONA_NO_REGISTRADA THEN
    p_sortida:='ERROR: persona no registrada - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO LOGBD(IdLog, IdProcediment, DataExecucio, Sortida)
           VALUES(SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

END Usuari;
/
