CREATE OR REPLACE PACKAGE DW AS
  PROCEDURE CarregaDW (p_setmana IN MDMOMENT.mSetmana%TYPE, p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_sortida OUT VARCHAR2);
END DW;
/

CREATE OR REPLACE PACKAGE BODY DW AS
/**********************************************************
 * Nom procediment: Carrega DW
 * Descripci:      carrega les dades al magatzem de dades
 **********************************************************/
PROCEDURE CarregaDW(p_setmana IN MDMOMENT.mSetmana%TYPE, p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_sortida OUT VARCHAR2) IS
  cntMoment       NUMBER;
  cntMDRegistre   NUMBER;
  cntMDVisita     NUMBER;
  cntMDIncidencia NUMBER;
  VidMoment       NUMBER;
  MOMENT_CARREGAT EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  sidRegistre     MDREGISTRE.idRegistre%TYPE;
  sidPersona      MDREGISTRE.idPersona%TYPE;
  snomEmpresa     MDREGISTRE.nomEmpresa%TYPE;
  snomDelegacio   MDREGISTRE.nomDelegacio%TYPE;
  shoraEntrada    MDREGISTRE.horaEntrada%TYPE;
  shoraSortida    MDREGISTRE.horaSortida%TYPE;
  sminutsRegistre MDREGISTRE.minutsRegistre%TYPE;
  sidVisita       MDVISITA.idVisita%TYPE;
  snomDepartament MDVISITA.nomDepartament%TYPE;
  snomEmpresaV    MDVISITA.nomEmpresa%TYPE;
  snomDelegacioV  MDVISITA.nomDelegacio%TYPE;
  smotiuVisita    MDVISITA.motiuVisita%TYPE;
  sidIncidencia    MDINCIDENCIA.idIncidencia%TYPE;
  snomCiutat       MDINCIDENCIA.nomCiutat%TYPE;
  snomPais         MDINCIDENCIA.nomPais%TYPE;
  stipusIncidencia MDINCIDENCIA.tipusIncidencia%TYPE;
  
  /* Cursor per llegir els registres segons el moment */
  CURSOR Registre(cSetmana NUMBER, cMes NUMBER, cAny NUMBER) IS
  SELECT SECURPFC.REGISTRE.idRegistre, SECURPFC.REGISTRE.ID, SECURPFC.REGISTRE.nomEmpresa, SECURPFC.REGISTRE.nomDelegacio, 
  SECURPFC.REGISTRE.horaEntrada, SECURPFC.REGISTRE.horaSortida, (SECURPFC.REGISTRE.horaSortida-SECURPFC.REGISTRE.horaEntrada)*24*60 AS VminutsRegistre
  FROM SECURPFC.REGISTRE WHERE to_number(to_char(SECURPFC.REGISTRE.horaEntrada,'IW')) = cSetmana AND 
  to_number(to_char(SECURPFC.REGISTRE.horaEntrada, 'mm')) = cMes AND to_number(to_char(SECURPFC.REGISTRE.horaEntrada, 'YYYY')) = cAny;

  /* Cursor per llegir les visites segons el moment */
  CURSOR Visita(cSetmana NUMBER, cMes NUMBER, cAny NUMBER) IS
  SELECT SECURPFC.VISITA.idVisita, SECURPFC.EMPLEAT.nomDepartament, SECURPFC.EMPLEAT.nomEmpresa, SECURPFC.EMPLEAT.nomDelegacio, SECURPFC.VISITA.motiuVisita
  FROM SECURPFC.VISITA, SECURPFC.EMPLEAT WHERE SECURPFC.VISITA.ID = SECURPFC.EMPLEAT.ID AND SECURPFC.VISITA.idEmpleat = SECURPFC.EMPLEAT.idEmpleat AND
  to_number(to_char(SECURPFC.VISITA.horaEntrada,'IW')) = cSetmana AND 
  to_number(to_char(SECURPFC.VISITA.horaEntrada, 'mm')) = cMes AND to_number(to_char(SECURPFC.VISITA.horaEntrada, 'YYYY')) = cAny;

  /* Cursor per llegir les incidencies segons el moment */
  CURSOR Incidencia(cSetmana NUMBER, cMes NUMBER, cAny NUMBER) IS
  SELECT SECURPFC.INCIDENCIA.idIncidencia, SECURPFC.DELEGACIO.nomCiutat, SECURPFC.DELEGACIO.nomPais, SECURPFC.INCIDENCIA.tipusIncidencia
  FROM SECURPFC.INCIDENCIA, SECURPFC.DELEGACIO WHERE SECURPFC.INCIDENCIA.nomEmpresa = SECURPFC.DELEGACIO.nomEmpresa AND SECURPFC.INCIDENCIA.nomDelegacio = DELEGACIO.nomDelegacio AND
  to_number(to_char(SECURPFC.INCIDENCIA.horaIncidencia,'IW')) = cSetmana AND 
  to_number(to_char(SECURPFC.INCIDENCIA.horaIncidencia, 'mm')) = cMes AND to_number(to_char(SECURPFC.INCIDENCIA.horaIncidencia, 'YYYY')) = cAny;

  BEGIN
  vProcedureName := 'CarregaDW';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);

  SELECT COUNT(1) INTO cntMoment FROM MDMOMENT WHERE mSetmana = p_setmana AND mMes = p_mes AND mAny = p_any;
  IF (cntMoment > 0) THEN
    RAISE MOMENT_CARREGAT;
  END IF;     
  VidMoment := SEQ_IDMOMENT.nextval;
  INSERT INTO MDMOMENT(idMoment, mSetmana, mMes, mAny) VALUES (VidMoment, p_setmana, p_mes, p_any);

  /* carrega les persones */
  INSERT INTO MDPERSONA(idPersona, dataNaixement, sexe)
  (SELECT SECURPFC.PERSONA.id, SECURPFC.PERSONA.dataNaixement, SECURPFC.PERSONA.sexe FROM SECURPFC.PERSONA WHERE 
  SECURPFC.PERSONA.id NOT IN (SELECT idPersona FROM MDPERSONA));

  /* carrega les delegacions */
  INSERT INTO MDDELEGACIO(nomEmpresa, nomDelegacio) 
  (SELECT SECURPFC.DELEGACIO.nomEmpresa, SECURPFC.DELEGACIO.nomDelegacio FROM SECURPFC.DELEGACIO WHERE
  (SECURPFC.DELEGACIO.nomEmpresa, SECURPFC.DELEGACIO.nomDelegacio) NOT IN (SELECT nomEmpresa, nomDelegacio FROM MDDELEGACIO));

  /* carrega les ciutats */
  INSERT INTO MDCIUTAT(nomCiutat, nomPais)(SELECT SECURPFC.CIUTAT.nomCiutat, SECURPFC.CIUTAT.nomPais FROM SECURPFC.CIUTAT WHERE (SECURPFC.CIUTAT.nomCiutat, SECURPFC.CIUTAT.nomPais) 
    NOT IN (SELECT nomCiutat, nomPais FROM MDCIUTAT));

  /* carrega els departaments */
  INSERT INTO MDDEPARTAMENT(nomDepartament, nomEmpresa, nomDelegacio) 
  (SELECT SECURPFC.DEPARTAMENT.nomDepartament, SECURPFC.DEPARTAMENT.nomEmpresa, SECURPFC.DEPARTAMENT.nomDelegacio FROM SECURPFC.DEPARTAMENT WHERE 
    (SECURPFC.DEPARTAMENT.nomDepartament, SECURPFC.DEPARTAMENT.nomEmpresa, SECURPFC.DEPARTAMENT.nomDelegacio) 
   NOT IN (SELECT nomDepartament, nomEmpresa, nomDelegacio FROM MDDEPARTAMENT));

  /* carrega el fet registre */
  OPEN Registre(p_setmana, p_mes, p_any);
  LOOP 
    FETCH Registre INTO sidRegistre, sidPersona, snomEmpresa, snomDelegacio, shoraEntrada, shoraSortida, sminutsRegistre;
    EXIT WHEN Registre%NOTFOUND;
    SELECT COUNT(1) INTO cntMDRegistre FROM MDREGISTRE WHERE idRegistre = sidRegistre;
    IF (cntMDRegistre > 0) THEN
      UPDATE MDREGISTRE SET idRegistre = sidRegistre, idMoment = VidMoment, idPersona = sidPersona, 
        nomEmpresa = snomEmpresa, nomDelegacio = snomDelegacio, horaEntrada = shoraEntrada, horaSortida = shoraSortida,
        minutsRegistre = sminutsRegistre WHERE idRegistre = sidRegistre;
    ELSE 
      INSERT INTO MDREGISTRE(idRegistre, idMoment, idPersona, nomEmpresa, nomDelegacio, horaEntrada, horaSortida, minutsRegistre) 
      VALUES (sidRegistre, VidMoment, sidPersona, snomEmpresa, snomDelegacio, shoraEntrada, shoraSortida, sminutsRegistre);
    END IF;                                                                                                            
  END LOOP;
  CLOSE Registre;

  /* carrega el fet visita */
  OPEN Visita(p_setmana, p_mes, p_any);
  LOOP 
    FETCH Visita INTO sidVisita, snomDepartament, snomEmpresaV, snomDelegacioV, smotiuVisita;
    EXIT WHEN Visita%NOTFOUND;
    SELECT COUNT(1) INTO cntMDVisita FROM MDVISITA WHERE idVisita = sidVisita;
    IF (cntMDVisita > 0) THEN
      UPDATE MDVISITA SET idVisita = sidVisita, idMoment = VidMoment, nomDepartament = snomDepartament, nomEmpresa = snomEmpresaV, nomDelegacio = snomDelegacioV, 
      motiuVisita = smotiuVisita WHERE idVisita = sidVisita;
    ELSE 
      INSERT INTO MDVISITA(idVisita, idMoment, nomDepartament, nomEmpresa, nomDelegacio, motiuVisita) 
      VALUES (sidVisita, VidMoment, snomDepartament, snomEmpresaV, snomDelegacioV, smotiuVisita);
    END IF;                                                                                                            
  END LOOP;
  CLOSE Visita;
 
 /* carrega el fet incidencia */
  OPEN Incidencia(p_setmana, p_mes, p_any);
  LOOP 
    FETCH Incidencia INTO sidIncidencia, snomCiutat, snomPais, stipusIncidencia;
    EXIT WHEN Incidencia%NOTFOUND;
    SELECT COUNT(1) INTO cntMDIncidencia FROM MDINCIDENCIA WHERE idIncidencia = sidIncidencia;
    IF (cntMDIncidencia > 0) THEN
      UPDATE MDINCIDENCIA SET idIncidencia = sidIncidencia, idMoment = VidMoment, nomCiutat = snomCiutat, nomPais = snomPais,
      tipusIncidencia = stipusIncidencia WHERE idIncidencia = sidIncidencia;
    ELSE 
      INSERT INTO MDINCIDENCIA(idIncidencia, idMoment, nomCiutat, nomPais, tipusIncidencia) 
      VALUES (sidIncidencia, VidMoment, snomCiutat, snomPais, stipusIncidencia);
    END IF;                                                                                                            
  END LOOP;
  CLOSE Incidencia;
 
  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN MOMENT_CARREGAT THEN
    p_sortida:='ERROR: moment existent - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN DUP_VAL_ON_INDEX THEN
    p_sortida:='ERROR: dades ja actualitzades - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR en carrega DW - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;
 
END DW;
/
