
/* Nous tipus necessaris per la sortida dels procediments del magatzem de dades */ 
CREATE OR REPLACE TYPE RecordCiutat AS OBJECT
       (nomCiutat VARCHAR2(60),
        nomPais   VARCHAR2(60));
/
CREATE OR REPLACE TYPE ArrayTopCiutatsIncidencies IS VARRAY(5) OF RecordCiutat;
/

CREATE OR REPLACE TYPE RecordIncidMes AS OBJECT
       (mMes NUMBER,
        numeroIncid NUMBER);
/
CREATE OR REPLACE TYPE ArrayIncidMes IS VARRAY(12) OF RecordIncidMes;
/

CREATE OR REPLACE PACKAGE USUARI_BI AS
  /* Consulta departament amb ms visites */
  PROCEDURE DepartamentMesVisites(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, 
                                  p_nomDepartament OUT MDVISITA.nomDepartament%TYPE, p_nomEmpresa OUT MDVISITA.nomEmpresa%TYPE,
                                  p_nomDelegacio OUT MDVISITA.nomDelegacio%TYPE, p_numVisites OUT NUMBER, p_sortida OUT VARCHAR2);
  /* Consulta ciutats amb ms incidncies (els robatoris sn un tipus d'incidncia) */
  PROCEDURE CiutatsMesIncidencies(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_tipusIncidencia IN VARCHAR2,
                                  p_CiutatsMesIncidencies IN OUT ArrayTopCiutatsIncidencies, p_sortida OUT VARCHAR2);
  /* Consulta mesos amb ms incidncies (quan es ms habitual que hi hagi robatoris, per mes) */
  PROCEDURE MesosMesIncidencies(p_any IN MDMOMENT.mAny%TYPE, p_tipusIncidencia IN VARCHAR2,
                                p_MesosMesIncidencies IN OUT ArrayIncidMes, p_sortida OUT VARCHAR2);
  /* Obt l'estadstica de registres d'entrada per edat */
  PROCEDURE HoresEntradaPerEdat(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaEdat OUT SYS_REFCURSOR, 
                                p_sortida OUT VARCHAR2);
  /* Obt l'estadstica de registres d'entrada per sexe */
  PROCEDURE HoresEntradaPerSexe(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaSexe OUT SYS_REFCURSOR, 
                                p_sortida OUT VARCHAR2);
  /* Obt l'estadstica de registres d'entrada per mesos */
  PROCEDURE HoresEntradaPerMesos(p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaMesos OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2);
END USUARI_BI;
/

CREATE OR REPLACE PACKAGE BODY USUARI_BI AS
/********************************************************
 * Nom procediment: DepartamentMesVisites
 * Descripci:      Consulta departament amb ms visites
 ********************************************************/
PROCEDURE DepartamentMesVisites(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, 
                                p_nomDepartament OUT MDVISITA.nomDepartament%TYPE, p_nomEmpresa OUT MDVISITA.nomEmpresa%TYPE,
                                p_nomDelegacio OUT MDVISITA.nomDelegacio%TYPE, p_numVisites OUT NUMBER, p_sortida OUT VARCHAR2) IS
                                
  DEPARTAMENT_NO_TROBAT EXCEPTION;
  CONSULTA_INCORRECTA   EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  
  CURSOR ConsultaDepartamentTot IS
    SELECT MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio, COUNT(1) AS NumVisites
    FROM MDVISITA GROUP BY MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio
    ORDER BY COUNT(1) DESC;
  CURSOR ConsultaDepartamentMesAny (c_mes NUMBER, c_any NUMBER) IS
    SELECT MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio, COUNT(1) AS NumVisites
    FROM MDVISITA, MDMOMENT WHERE MDVISITA.idMoment = MDMOMENT.idMoment AND
    MDMOMENT.mMes = c_mes AND MDMOMENT.mAny = c_any
    GROUP BY MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio
    ORDER BY COUNT(1) DESC;
  CURSOR ConsultaDepartamentAny (c_any NUMBER) IS
    SELECT MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio, COUNT(1) AS NumVisites
    FROM MDVISITA, MDMOMENT WHERE MDVISITA.idMoment = MDMOMENT.idMoment AND MDMOMENT.mAny = c_any
    GROUP BY MDVISITA.nomDepartament, MDVISITA.nomEmpresa, MDVISITA.nomDelegacio
    ORDER BY COUNT(1) DESC;

BEGIN
  vProcedureName := 'DepartamentMesVisites';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);

  IF ( (p_mes = 0) AND (p_any = 0) ) THEN 
    OPEN ConsultaDepartamentTot;
    FETCH ConsultaDepartamentTot INTO p_nomDepartament, p_nomEmpresa, p_nomDelegacio, p_numVisites;
    IF ConsultaDepartamentTot%NOTFOUND THEN
      RAISE DEPARTAMENT_NO_TROBAT;
    END IF;                                                                                                           
    CLOSE ConsultaDepartamentTot;
  ELSE
    IF ( (p_mes > 0) AND (p_any > 0) ) THEN 
      OPEN ConsultaDepartamentMesAny(p_mes, p_any);
      FETCH ConsultaDepartamentMesAny INTO p_nomDepartament, p_nomEmpresa, p_nomDelegacio, p_numVisites;
      IF ConsultaDepartamentMesAny%NOTFOUND THEN
        RAISE DEPARTAMENT_NO_TROBAT;
      END IF;                                                                                                           
      CLOSE ConsultaDepartamentMesAny;
    ELSE
      IF ( (p_mes = 0) AND (p_any > 0) ) THEN 
        OPEN ConsultaDepartamentAny(p_any);
        FETCH ConsultaDepartamentAny INTO p_nomDepartament, p_nomEmpresa, p_nomDelegacio, p_numVisites;
        IF ConsultaDepartamentAny%NOTFOUND THEN
          RAISE DEPARTAMENT_NO_TROBAT;
        END IF;                                                                                                           
        CLOSE ConsultaDepartamentAny;
      ELSE
        RAISE CONSULTA_INCORRECTA;
      END IF;
    END IF;
  END IF;

  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN DEPARTAMENT_NO_TROBAT THEN
    p_sortida:='ERROR: departament no trobat - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  
  WHEN CONSULTA_INCORRECTA THEN
    p_sortida:='ERROR: parametres consulta incorrectes - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**************************************************************************************************
 * Nom procediment: CiutatsMesIncidencies
 * Descripci:      Consulta ciutats amb ms incidncies (els robatoris sn un tipus d'incidncia)
 **************************************************************************************************/
PROCEDURE CiutatsMesIncidencies(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_tipusIncidencia IN VARCHAR2,
                                p_CiutatsMesIncidencies IN OUT ArrayTopCiutatsIncidencies, p_sortida OUT VARCHAR2) IS
                                
  CONSULTA_INCORRECTA   EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  vCiutat VARCHAR2(60);
  vPais VARCHAR2(60);
  i NUMBER;
  vNumIncidencies NUMBER;
  
  CURSOR ConsultaIncidCiutat IS
    SELECT MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais, COUNT(1) AS NumIncidencies
    FROM MDINCIDENCIA WHERE MDINCIDENCIA.tipusIncidencia = p_tipusIncidencia
    GROUP BY MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais
    ORDER BY COUNT(1) DESC;
 
  CURSOR ConsultaIncidCiutatMesAny (c_mes MDMOMENT.mMes%TYPE, c_any MDMOMENT.mAny%TYPE) IS
    SELECT MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais, COUNT(1) AS NumIncidencies
    FROM MDINCIDENCIA, MDMOMENT WHERE MDINCIDENCIA.idMoment = MDMOMENT.idMoment AND 
    MDINCIDENCIA.tipusIncidencia = p_tipusIncidencia AND MDMOMENT.mMes = c_mes AND MDMOMENT.mAny = c_any
    GROUP BY MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais
    ORDER BY COUNT(1) DESC;
 
  CURSOR ConsultaIncidCiutatAny (c_any MDMOMENT.mMes%TYPE) IS
    SELECT MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais, COUNT(1) AS NumIncidencies
    FROM MDINCIDENCIA, MDMOMENT WHERE MDINCIDENCIA.idMoment = MDMOMENT.idMoment AND 
    MDINCIDENCIA.tipusIncidencia = p_tipusIncidencia AND MDMOMENT.mAny = c_any
    GROUP BY MDINCIDENCIA.nomCiutat, MDINCIDENCIA.nomPais
    ORDER BY COUNT(1) DESC;
   
BEGIN
  vProcedureName := 'CiutatsMesIncidencies';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  i := 1; 
  IF ( (p_mes = 0) AND (p_any = 0) ) THEN 
    OPEN ConsultaIncidCiutat;
    LOOP
      FETCH ConsultaIncidCiutat 
      INTO vCiutat, vPais, vNumIncidencies;
      EXIT WHEN ConsultaIncidCiutat%NOTFOUND OR i > 5;
      p_CiutatsMesIncidencies.extend;
      p_CiutatsMesIncidencies(i) := RecordCiutat(vCiutat, vPais);
      i := i + 1;
    END LOOP;
    CLOSE ConsultaIncidCiutat;
  ELSE
    IF ( (p_mes > 0) AND (p_any > 0) ) THEN 
      OPEN ConsultaIncidCiutatMesAny(p_mes, p_any);
      LOOP
        FETCH ConsultaIncidCiutatMesAny 
        INTO vCiutat, vPais, vNumIncidencies;
        EXIT WHEN ConsultaIncidCiutatMesAny%NOTFOUND OR i > 5;
        p_CiutatsMesIncidencies.extend;
        p_CiutatsMesIncidencies(i) := RecordCiutat(vCiutat, vPais);
        i := i + 1;
      END LOOP;                                                                                                           
      CLOSE ConsultaIncidCiutatMesAny;
    ELSE
      IF ( (p_mes = 0) AND (p_any > 0) ) THEN 
        OPEN ConsultaIncidCiutatAny(p_any);
        LOOP
          FETCH ConsultaIncidCiutatAny 
          INTO vCiutat, vPais, vNumIncidencies;
          EXIT WHEN ConsultaIncidCiutatAny%NOTFOUND OR i > 5;
          p_CiutatsMesIncidencies.extend;
          p_CiutatsMesIncidencies(i) := RecordCiutat(vCiutat, vPais);
          i := i + 1;
        END LOOP;                                                                                                           
        CLOSE ConsultaIncidCiutatAny;
      ELSE
        RAISE CONSULTA_INCORRECTA;
      END IF;
    END IF;
  END IF;

 INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
       VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN CONSULTA_INCORRECTA THEN
    p_sortida:='ERROR: parametres consulta incorrectes - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));

  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/******************************************************
 * Nom procediment: MesosMesIncidencies
 * Descripci:      Consulta mesos amb ms incidncies
 ******************************************************/
PROCEDURE MesosMesIncidencies(p_any IN MDMOMENT.mAny%TYPE, p_tipusIncidencia IN VARCHAR2,
                              p_MesosMesIncidencies IN OUT ArrayIncidMes, p_sortida OUT VARCHAR2) IS
  CONSULTA_INCORRECTA   EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
  vMes NUMBER;
  vNumIncidencies NUMBER;
  i NUMBER;
  
  CURSOR ConsultaIncidMesos IS
    SELECT MDMOMENT.mMes, COUNT(1) AS NumIncidencies
    FROM MDINCIDENCIA, MDMOMENT WHERE MDINCIDENCIA.idMoment = MDMOMENT.idMoment AND
    MDINCIDENCIA.tipusIncidencia = p_tipusIncidencia
    GROUP BY MDMOMENT.mMes
    ORDER BY COUNT(1) DESC;
 
  CURSOR ConsultaIncidMesosAny (c_any NUMBER) IS
    SELECT MDMOMENT.mMes, COUNT(1) AS NumIncidencies
    FROM MDINCIDENCIA, MDMOMENT WHERE MDINCIDENCIA.idMoment = MDMOMENT.idMoment AND 
    MDINCIDENCIA.tipusIncidencia = p_tipusIncidencia AND MDMOMENT.mAny = c_any
    GROUP BY MDMOMENT.mMes
    ORDER BY COUNT(1) DESC;
   
BEGIN
  vProcedureName := 'MesosMesIncidencies';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  i := 1;
 
  IF (p_any = 0) THEN 
    OPEN ConsultaIncidMesos;
    LOOP
      FETCH ConsultaIncidMesos 
      INTO vMes, vNumIncidencies;
      EXIT WHEN ConsultaIncidMesos%NOTFOUND OR i > 12;
      p_MesosMesIncidencies.extend;
      p_MesosMesIncidencies(i) := RecordIncidMes(vMes, vNumIncidencies);
      i := i + 1;
    END LOOP;
    CLOSE ConsultaIncidMesos;
  ELSE
    OPEN ConsultaIncidMesosAny(p_any);
    LOOP
      FETCH ConsultaIncidMesosAny 
      INTO vMes, vNumIncidencies;
      EXIT WHEN ConsultaIncidMesosAny%NOTFOUND OR i > 12;
      p_MesosMesIncidencies.extend;
      p_MesosMesIncidencies(i) := RecordIncidMes(vMes, vNumIncidencies);
      i := i + 1;
    END LOOP;                                                                                                           
    CLOSE ConsultaIncidMesosAny;
  END IF;

  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN CONSULTA_INCORRECTA THEN
    p_sortida:='ERROR: parametres consulta incorrectes - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));

  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**********************************************************************
 * Nom procediment: HoresEntradaPerEdat
 * Descripci:      Obt l'estadstica de registres d'entrada per edat
 **********************************************************************/ 
PROCEDURE HoresEntradaPerEdat(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaEdat OUT SYS_REFCURSOR, 
                              p_sortida OUT VARCHAR2) IS
                                
  CONSULTA_INCORRECTA   EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
     
BEGIN
  vProcedureName := 'HoresEntradaPerEdat';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  IF ( (p_mes = 0) AND (p_any = 0) ) THEN 
    OPEN p_HoresEntradaEdat FOR
      SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
      EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement) AS Edat, COUNT(1) AS NumRegistres
      FROM MDREGISTRE, MDPERSONA WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona
      GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
      EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement)
      ORDER BY EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement), COUNT(1);
--      CLOSE p_HoresEntradaEdat;
  ELSE
    IF ( (p_mes > 0) AND (p_any > 0) ) THEN 
      OPEN p_HoresEntradaEdat FOR
        SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
        EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement) AS Edat, COUNT(1) AS NumRegistres
        FROM MDREGISTRE, MDPERSONA, MDMOMENT WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona AND MDREGISTRE.idMoment = MDMOMENT.idMoment AND
        MDMOMENT.mMes = p_mes AND MDMOMENT.mAny = p_any
        GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
        EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement)
        ORDER BY EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement), COUNT(1);
  --    CLOSE p_HoresEntradaEdat;
    ELSE
      IF ( (p_mes = 0) AND (p_any > 0) ) THEN 
        OPEN p_HoresEntradaEdat FOR                                                                                                  
          SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
          EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement) AS Edat, COUNT(1) AS NumRegistres
          FROM MDREGISTRE, MDPERSONA, MDMOMENT WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona AND MDREGISTRE.idMoment = MDMOMENT.idMoment AND
          MDMOMENT.mAny = p_any
          GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
          EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement)
          ORDER BY EXTRACT(YEAR FROM sysdate) - EXTRACT(YEAR FROM MDPERSONA.dataNaixement), COUNT(1);
    --    CLOSE p_HoresEntradaEdat;
      ELSE
        RAISE CONSULTA_INCORRECTA;
      END IF;
    END IF;
  END IF;

  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN CONSULTA_INCORRECTA THEN
    p_sortida:='ERROR: parametres consulta incorrectes - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));

  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/**********************************************************************
 * Nom procediment: HoresEntradaPerSexe
 * Descripci:      Obt l'estadstica de registres d'entrada per sexe
 **********************************************************************/   
PROCEDURE HoresEntradaPerSexe(p_mes IN MDMOMENT.mMes%TYPE, p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaSexe OUT SYS_REFCURSOR, 
                              p_sortida OUT VARCHAR2) IS
                                
  CONSULTA_INCORRECTA   EXCEPTION;
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
     
BEGIN
  vProcedureName := 'HoresEntradaPerSexe';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  IF ( (p_mes = 0) AND (p_any = 0) ) THEN 
    OPEN p_HoresEntradaSexe FOR
      SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
      MDPERSONA.sexe , COUNT(1) AS NumRegistres
      FROM MDREGISTRE, MDPERSONA WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona
      GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
      MDPERSONA.sexe
      ORDER BY MDPERSONA.sexe, COUNT(1);
   --   CLOSE p_HoresEntradaSexe;
  ELSE
    IF ( (p_mes > 0) AND (p_any > 0) ) THEN 
      OPEN p_HoresEntradaSexe FOR
        SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
        MDPERSONA.sexe, COUNT(1) AS NumRegistres
        FROM MDREGISTRE, MDPERSONA, MDMOMENT WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona AND MDREGISTRE.idMoment = MDMOMENT.idMoment AND
        MDMOMENT.mMes = p_mes AND MDMOMENT.mAny = p_any
        GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
        MDPERSONA.sexe
        ORDER BY MDPERSONA.sexe, COUNT(1);
   --   CLOSE p_HoresEntradaSexe;
    ELSE
      IF ( (p_mes = 0) AND (p_any > 0) ) THEN 
        OPEN p_HoresEntradaSexe FOR                                                                                                  
          SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
          MDPERSONA.sexe, COUNT(1) AS NumRegistres
          FROM MDREGISTRE, MDPERSONA, MDMOMENT WHERE MDREGISTRE.idPersona = MDPERSONA.idPersona AND MDREGISTRE.idMoment = MDMOMENT.idMoment AND
          MDMOMENT.mAny = p_any
          GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
          MDPERSONA.sexe
          ORDER BY MDPERSONA.sexe, COUNT(1);
   --     CLOSE p_HoresEntradaSexe;
      ELSE
        RAISE CONSULTA_INCORRECTA;
      END IF;
    END IF;
  END IF;

  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN CONSULTA_INCORRECTA THEN
    p_sortida:='ERROR: parametres consulta incorrectes - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK; 
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));

  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

/***********************************************************************
 * Nom procediment: HoresEntradaPerMesos
 * Descripci:      Obt l'estadstica de registres d'entrada per mesos
 ***********************************************************************/ 
PROCEDURE HoresEntradaPerMesos(p_any IN MDMOMENT.mAny%TYPE, p_HoresEntradaMesos OUT SYS_REFCURSOR, p_sortida OUT VARCHAR2) IS
                                
  status    VARCHAR2(50 CHAR);
  vDebugProc VARCHAR2(50 CHAR);
  vProcedureName VARCHAR2(30);
     
BEGIN
  vProcedureName := 'HoresEntradaPerMesos';
  p_sortida := 'OK';
  VDebugProc := $$PLSQL_UNIT;
  DBMS_OUTPUT.PUT_LINE(VDebugProc);
 
  IF (p_any = 0) THEN 
    OPEN p_HoresEntradaMesos FOR
      SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
      MDMOMENT.mMes, COUNT(1) AS NumRegistres
      FROM MDREGISTRE, MDMOMENT WHERE MDREGISTRE.idMoment = MDMOMENT.idMoment
      GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
      MDMOMENT.mMes
      ORDER BY MDMOMENT.mMes, COUNT(1);
--      CLOSE p_HoresEntradaMesos;
  ELSE
    OPEN p_HoresEntradaMesos FOR                                                                                                  
      SELECT to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI') AS nearest_15_min,
      MDMOMENT.mMes, COUNT(1) AS NumRegistres
      FROM MDREGISTRE, MDMOMENT WHERE MDREGISTRE.idMoment = MDMOMENT.idMoment AND MDMOMENT.mAny = p_any
      GROUP BY to_CHAR(trunc(MDREGISTRE.horaEntrada, 'hh') + CEIL((to_char(MDREGISTRE.horaEntrada, 'mi') + (to_char(MDREGISTRE.horaEntrada, 'ss')/60))/15)*15/1440, 'HH24:MI'),
      MDMOMENT.mMes
      ORDER BY MDMOMENT.mMes, COUNT(1);
--    CLOSE p_HoresEntradaMesos;
  END IF;

  INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
         VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, p_sortida);
EXCEPTION
  WHEN OTHERS THEN
    p_sortida:='ERROR - Line '||$$PLSQL_LINE||' Error:'||SQLCODE||'-'||SUBSTR(SQLERRM, 1, 100);
    DBMS_OUTPUT.PUT_LINE(p_sortida);
    DBMS_OUTPUT.PUT_LINE(VDebugProc||'.'||vProcedureName);
    ROLLBACK;
    INSERT INTO SECURPFC.LOGBD(SECURPFC.LOGBD.IdLog, SECURPFC.LOGBD.IdProcediment, SECURPFC.LOGBD.DataExecucio, SECURPFC.LOGBD.Sortida)
           VALUES(SECURPFC.SEQ_LOGID.nextval, vDebugProc||'.'||vProcedureName, SYSDATE, substr(p_sortida,1,150));
END;

END USUARI_BI;
/
